@@ -287,12 +287,18 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
     MAKE_SIGNAL(NV_CTRL_THERMAL_COOLER_LEVEL_SET_DEFAULT);
     MAKE_SIGNAL(NV_CTRL_THERMAL_COOLER_CONTROL_TYPE);
     MAKE_SIGNAL(NV_CTRL_THERMAL_COOLER_TARGET);
+    MAKE_SIGNAL(NV_CTRL_GPU_ECC_CONFIGURATION);
     MAKE_SIGNAL(NV_CTRL_GPU_POWER_MIZER_MODE);
     MAKE_SIGNAL(NV_CTRL_GVI_SYNC_OUTPUT_FORMAT);
     MAKE_SIGNAL(NV_CTRL_GVI_MAX_CHANNELS_PER_JACK);
     MAKE_SIGNAL(NV_CTRL_GVI_MAX_STREAMS);
     MAKE_SIGNAL(NV_CTRL_GVI_NUM_CAPTURE_SURFACES);
     MAKE_SIGNAL(NV_CTRL_OVERSCAN_COMPENSATION);
+    MAKE_SIGNAL(NV_CTRL_GPU_PCIE_GENERATION);
+    MAKE_SIGNAL(NV_CTRL_GVI_BOUND_GPU);
+    MAKE_SIGNAL(NV_CTRL_ACCELERATE_TRAPEZOIDS);
+    MAKE_SIGNAL(NV_CTRL_GPU_CORES);
+    MAKE_SIGNAL(NV_CTRL_GPU_MEMORY_BUS_WIDTH);
 
 #undef MAKE_SIGNAL
     
@@ -303,7 +309,7 @@ static void ctk_event_class_init(CtkEventClass *ctk_event_class)
      * knows about.
      */
 
-#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_OVERSCAN_COMPENSATION
+#if NV_CTRL_LAST_ATTRIBUTE != NV_CTRL_GPU_MEMORY_BUS_WIDTH
 #warning "There are attributes that do not emit signals!"
 #endif
 
